use crate::Day;

pub struct Day02(Vec<Instruction>);

enum Instruction {
    Forward(i32),
    Down(i32),
    Up(i32),
}

impl Day for Day02 {
    fn init(content: String) -> Self {
        let lines = content.lines().map(str_to_instr).collect::<Vec<Instruction>>();
        Day02(lines)
    }

    fn part1(&self) -> String {
        let mut depth = 0;
        let mut fw = 0;
        for i in &self.0 {
            match i {
                Instruction::Forward(n) => fw += n,
                Instruction::Down(n) => depth += n,
                Instruction::Up(n) => depth -= n,
            }
        }
        format!("{}", depth * fw)
    }

    fn part2(&self) -> String {
        let mut depth = 0;
        let mut fw = 0;
        let mut aim = 0;
        for i in &self.0 {
            match i {
                Instruction::Forward(n) => {
                    fw += n;
                    depth += aim * n;
                }
                Instruction::Down(n) => aim += n,
                Instruction::Up(n) => aim -= n,
            }
        }
        format!("{}", depth * fw)
    }
}

fn str_to_instr(s: &str) -> Instruction {
    let v = s.split(' ').collect::<Vec<&str>>();
    let name = v[0];
    let height = v[1].parse().expect("This should be an int :p");
    match name {
        "forward" => Instruction::Forward(height),
        "up" => Instruction::Up(height),
        "down" => Instruction::Down(height),
        _ => panic!("How did this happen???")
    }
}

#[cfg(test)]
mod tests {
    use std::fs::read_to_string;
    use crate::day02::Day02;
    use crate::day::Day;

    const D: &str = r"forward 5
down 5
forward 8
up 3
down 8
forward 2";

    #[test]
    fn part1_test() {
        let d2 = Day02::init(String::from(D));
        assert_eq!("150", d2.part1())
    }

    #[test]
    fn part2_test() {
        let d2 = Day02::init(String::from(D));
        assert_eq!("900", d2.part2())
    }

    #[test]
    pub fn part1_real() {
        let f = read_to_string("./input/day02").expect("Could not load input");
        let d = Day02::init(f);
        assert_eq!("1524750", d.part1())
    }

    #[test]
    pub fn part2_real() {
        let f = read_to_string("./input/day02").expect("Could not load input");
        let d = Day02::init(f);
        assert_eq!("1592426537", d.part2())
    }

    #[test]
    #[should_panic]
    fn very_invalid_input(){
        let f = String::from("invalid 10");
        Day02::init(f);
    }
}